//
//  CloudViewController.m
//  MicroLifeDeviceSDK-Dome
//
//  Created by willy.wu on 2019/10/9.
//  Copyright © 2019 WiFi@MBP. All rights reserved.
//

#import "CloudViewController.h"

@interface CloudViewController () <BPMDataResponseDelegate>

@property (strong, nonatomic) BPMBLEManager *aBPMBLEManager;

@property (strong, nonatomic) NSString *code;
@property (strong, nonatomic) NSString *OAuthTOKEN;
@property (strong, nonatomic) NSString *RefreashTOKEN;
@property (strong, nonatomic) NSString *bp_device_macAddres;
@property (strong, nonatomic) NSString *BPMuserID;
@property (assign, nonatomic) BOOL isAfib;

@end

@implementation CloudViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.code = [WxySystemTool xy_ReadNSUserDefaults:@"code"];
    self.OAuthTOKEN = [WxySystemTool xy_ReadNSUserDefaults:@"access_token"];
    self.RefreashTOKEN = [WxySystemTool xy_ReadNSUserDefaults:@"refresh_token"];
    NSString *log = [NSString stringWithFormat:@"code:%@\n\naccess_token:%@\n\nrefresh_token:%@",self.code,self.OAuthTOKEN,self.RefreashTOKEN];
    [self addLogWhitText:log];
    
    self.aBPMBLEManager = [BPMBLEManager shareInstanceWhithAuthorizationkey:SDKkey_BPM];
    self.aBPMBLEManager.dataResponseDelegate = self;
    
}

- (void)getMemberData {
    [MicroLifeCloudManager postGetMemberData:self.OAuthTOKEN client_id:Client_id client_secret:Client_secret FinishBlock:^(bool success, NSInteger code, MicroLifeCloudManager * _Nonnull microLifeCloudManager) {
        [self clearLogStr];
        NSString *log = [NSString stringWithFormat:@"getMemberData:\n%@",microLifeCloudManager.data];
        [self addLogWhitTextInTop:log];
    }];
}

- (void)getBPMHistoryData {
    [MicroLifeCloudManager postGetBPMHistoryData:self.OAuthTOKEN FinishBlock:^(bool success, NSInteger code, MicroLifeCloudManager * _Nonnull microLifeCloudManager) {
        NSString *log = [NSString stringWithFormat:@"getBPMHistoryData:\n%@",microLifeCloudManager.data];
        [self addLogWhitTextInTop:log];
    }];
}

- (void)addBPM:(MicroLifeCurrentAndMData *)mData {
    [self clearLogStr];
    NSString *date = [NSString stringWithFormat:@"%d-%02d-%02d %02d:%02d:00",2000+mData.year,mData.month,mData.day,mData.hour,mData.minute];
    int AFIB = 0;
    int PAD = 0;
    int MAM = mData.arr;
    if (mData.isFor3G) {
        PAD = self.isAfib?0:MAM;
        AFIB = self.isAfib?MAM:0;
    }else {
        PAD = mData.IHB;
        AFIB = mData.AFIb;
    }
    NSString *log = [NSString stringWithFormat:@"BPMBLEManagerResponseReadHistory:%@",mData.dictionaryFromModel];
    [self addLogWhitText:log];
    
    kWeakSelf(self)
    [MicroLifeCloudManager postAddBPM:self.OAuthTOKEN user_id:self.BPMuserID sys:(float)mData.sys dia:(float)mData.dia pul:mData.hr cuffokr:mData.cuffokr bpm_id:@"1" afib:AFIB pad:PAD man:MAM date:date mac_address:self.bp_device_macAddres FinishBlock:^(bool success, NSInteger code, MicroLifeCloudManager * _Nonnull microLifeCloudManager) {
        NSString *log = [NSString stringWithFormat:@"Upload cloud:%@\nDetailed response:%@",success?@"Yes":@"No",microLifeCloudManager.dictionaryFromModel];
        [weakself addLogWhitText:log];
    }];
}

- (IBAction)logOutAction:(id)sender {
    [WxySystemTool xy_RemoveAllDefaultData];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"logoutWork" object:nil];
    [self dismissViewControllerAnimated:YES completion:nil];
}
 
- (IBAction)workingAction:(id)sender {
    [self clearLogStr];
    switch ([sender tag]) {
        case 100:
            [self getMemberData];
            break;
        case 101:
            [self getBPMHistoryData];
            break;
        default:
            break;
    }
}

#pragma mark - BPMDataResponseDelegate
- (void)BPMBLEManagerCellPhoneBluetoothDidUpdateState:(MicroLifeBLEState)state {
    if (state == MicroLifeBLEStatePoweredOn) {
        [self addLogWhitText:@"BPMBLEManagerCellPhoneBluetoothDidUpdateState:MicroLifeBLEStatePoweredOn"];
    }else {
        [self addLogWhitText:@"BPMBLEManagerCellPhoneBluetoothDidUpdateState:MicroLifeBLEStatePoweredOff"];
    }
}

- (void)BPMBLEManagerDidConnectDevice {
    NSString *macAddress = [self.aBPMBLEManager.bindingDevice.macAddress debugDescription];
    macAddress = [macAddress stringByReplacingOccurrencesOfString:@"<" withString:@""];
    macAddress = [macAddress stringByReplacingOccurrencesOfString:@" " withString:@""];
    macAddress = [macAddress stringByReplacingOccurrencesOfString:@">" withString:@""];
    WxyStringTool *hsMacAddress = [WxyStringTool xy_InitWithStr:macAddress];
    self.bp_device_macAddres = @"";
    while (hsMacAddress.inputString.length > 0) {
        NSString *macbit = [hsMacAddress cutStr:2];
        self.bp_device_macAddres = [self.bp_device_macAddres stringByAppendingFormat:@"%@%@", macbit,(hsMacAddress.inputString.length > 0)?@":":@""];
    }
}

- (void)BPMBLEManagerDidDisconnectDevice {
    [self.aBPMBLEManager reScan];
}

- (void)BPMBLEManagerDidDiscoverBluetoothDeviceMacAddress:(NSData *)macAddress Name:(NSString *)name RSSI:(NSNumber *)RSSI {
    [self.aBPMBLEManager bindingDevice:macAddress];
}

- (void)BPMBLEManagerDidFailToConnectDevice {
    [self.aBPMBLEManager reScan];
}

- (void)BPMBLEManagerResponseReadHistory:(MicroLifeDRecord *)data {
    MicroLifeCurrentAndMData *mData = [data.MData lastObject];
    [self addBPM:mData];
    [self.aBPMBLEManager disconnect];
}

- (void)BPMBLEManagerResponseReadUserAndVersionData:(MicroLifeUserInfo *)user VersionData:(MicroLifeDeviceInfo *)verData {
    self.BPMuserID = [user.BPMuserID substringToIndex:11];
    self.isAfib = verData.optionAfib;
    NSString *log = [NSString stringWithFormat:@"BPMBLEManagerResponseReadUser:%@",self.BPMuserID];
    [self addLogWhitText:log];    
}

- (void)BPMBLEManagerResponseWriteDeviceTime:(BOOL)isSuccess {
    [self.aBPMBLEManager readHistorys];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
