//
//  RootViewController.m
//  MicroLifeDeviceSDK-Dome
//
//  Created by WiFi@ on 2018/5/21.
//  Copyright © 2018年 WiFi@MBP. All rights reserved.
//

#import "RootViewController.h"
#import "CloudViewController.h"

@interface RootViewController ()
@property (strong, nonatomic) NSString *logStr;
@property (weak, nonatomic) IBOutlet UIButton *cloudAPIBut;

@end

@implementation RootViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Do any additional setup after loading the view.
    NSString *appVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    NSString *appBVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
    self.VersionNumber.text = [NSString stringWithFormat:@"V %@(%@)",appVersion,appBVersion];
    
    self.MicroLifeDeviceSDKVersionNumber.text = [NSString stringWithFormat:@"DeviceSDKVersionNumber：%.1f",MicroLifeDeviceSDKVersionNumber];
    
    self.logView.editable = NO;
    self.logView.layoutManager.allowsNonContiguousLayout = NO;
        
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(logoutWork) name:@"logoutWork" object:nil];

}

- (void)logoutWork {
    if ([WxySystemTool xy_ReadNSUserDefaults:@"code"]) {
        self.cloudAPIBut.alpha = 1.0;
    } else {
        self.cloudAPIBut.alpha = 0.5;
    }
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self logoutWork];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (MicroLifeUserInfo *)currUser {
    if (!_currUser) {
        MicroLifeUserInfo *user = [MicroLifeUserInfo new];
        user.EBodyuserID = [self getEBodyuserID];
        user.BPMuserID = [self getBPMuserID];
        user.UBuserID = [self getUBuserID];
        user.resistance = 0;
        user.gender = arc4random_uniform(2)?MicroLifeUserGenderFemale:MicroLifeUserGenderMale;
        user.age = 18+arc4random_uniform(62);
        user.weight = 20+arc4random_uniform(130);
        user.height = 100+arc4random_uniform(120);
        user.roleType = arc4random_uniform(2)?MicroLifeUserRoleTypeSportsman:MicroLifeUserRoleTypeNormal;
        _currUser = user;
    }
    return _currUser;
}

- (NSString *)logStr {
    if (!_logStr) {
        self.logStr = [NSString new];
    }
    return _logStr;
}

- (void)clearLogStr {
    self.logStr = @"";
    dispatch_async(dispatch_get_main_queue(), ^{
        self.logView.text = self.logStr;
        [self.logView scrollRangeToVisible:NSMakeRange(0, self.logStr.length)];
    });
}

- (void)addLogWhitText:(NSString *)text {
    [self.view endEditing:true];
    NSLog(@"\n%@",text);
    NSDateFormatter *dFormatter = [NSDateFormatter new];
    [dFormatter setLocale:[NSLocale currentLocale]];
    [dFormatter setTimeZone:[NSTimeZone systemTimeZone]];
    [dFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss:SSS"];
    NSString *date = [dFormatter stringFromDate:[NSDate date]];
    self.logStr = [self.logStr stringByAppendingFormat:@"%@\n%@\n***\n",date,text];
    dispatch_async(dispatch_get_main_queue(), ^{
        self.logView.text = self.logStr;
        [self.logView scrollRangeToVisible:NSMakeRange(0, self.logStr.length)];
    });
}

- (void)addLogWhitTextInTop:(NSString *)text {
    NSLog(@"%@",text);
    [self clearLogStr];
    NSDateFormatter *dFormatter = [NSDateFormatter new];
    [dFormatter setLocale:[NSLocale currentLocale]];
    [dFormatter setTimeZone:[NSTimeZone systemTimeZone]];
    [dFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss:SSS"];
    NSString *date = [dFormatter stringFromDate:[NSDate date]];
    self.logStr = [self.logStr stringByAppendingFormat:@"%@\n%@\n***\n",date,text];
    dispatch_async(dispatch_get_main_queue(), ^{
        self.logView.text = self.logStr;
    });
}

- (void)bindingDeviceWhitMacAddres:(NSData *)macAddress {
    self.bindingDevice.text = [NSString stringWithFormat:@"Did Discover Bluetooth Device：%@",macAddress.debugDescription];
    self.bindingDevice.adjustsFontSizeToFitWidth = YES;
}

- (NSString *)getBPMuserID {
    NSMutableString* BPMuserID = [[NSMutableString alloc] init ];
    for (int i = 0; i < 11; i++) {
        if(i < 9){
            [BPMuserID appendString:[NSString stringWithFormat:@"%d", arc4random_uniform(10)]];
        }
        else{
            NSString *str = @"QWERTYUIOPASDFGHJKLZXCVBNM";
            [BPMuserID appendString:[NSString stringWithFormat:@"%@",[str substringWithRange:NSMakeRange(arc4random_uniform(26), 1)]]];
        }
    }
    return BPMuserID;
}

- (NSString *)getEBodyuserID {
    NSMutableString* EBodyuserID = [[NSMutableString alloc] init ];
    for (int i = 0; i < 11; i++) {
        [EBodyuserID appendString:[NSString stringWithFormat:@"%d", arc4random_uniform(10)]];
    }
    return EBodyuserID;
}

- (NSString *)getUBuserID {
    NSMutableString* UBuserID = [[NSMutableString alloc] init ];
    for (int i = 0; i < 5; i++) {
        [UBuserID appendString:[NSString stringWithFormat:@"%d", arc4random_uniform(10)]];
    }
    return UBuserID;
}

- (IBAction)CloudAPIAction:(id)sender {
    NSString *code = [WxySystemTool xy_ReadNSUserDefaults:@"code"];
    if (code) {
        if ([WxySystemTool xy_ReadNSUserDefaults:@"access_token"]) {
            [self goCloudVC];
        } else {
            kWeakSelf(self)
            [MicroLifeCloudManager postOAuthToken:authorization_code code:code refreshToken:@"" clientID:Client_id clientSecret:Client_secret redirectURI:Redirect_uri FinishBlock:^(bool success, NSInteger code, MicroLifeCloudManager * _Nonnull microLifeCloudManager) {
                if (success) {
                    [weakself goCloudVC];
                    [WxySystemTool xy_SaveNSUserDefaults:microLifeCloudManager.access_token Key:@"access_token"];
                    [WxySystemTool xy_SaveNSUserDefaults:microLifeCloudManager.refresh_token Key:@"refresh_token"];
                } else {
                    [weakself goCloudLogin];
                }
            }];
        }
    } else {
        [self goCloudLogin];
    }
}

- (void)goCloudLogin {
    [WxySystemTool xy_RemoveAllDefaultData];
    [MicroLifeCloudManager loginWithVC:self ClientId:Client_id RedirectURL:Redirect_uri Lang:nil animated:true completion:^{
    }];
}

- (void)goCloudVC {
    UIStoryboard *story = [UIStoryboard storyboardWithName:@"Main" bundle: nil];
    CloudViewController *cloudVC = [story instantiateViewControllerWithIdentifier:@"CloudVC"];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self presentViewController:cloudVC animated:YES completion:nil];
    });
}



/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
